SystemOrganization addCategory: #ZTimestamp!

Magnitude subclass: #ZTimestamp
	instanceVariableNames: 'jdn secs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestamp commentStamp: 'SvenVanCaekenberghe 4/20/2012 22:55' prior: 0!
I am ZTimestamp.
I am a Magnitude.
I represent a point in time, a combination of a date and a time.

I am an alternative for DateAndTime and TimeStamp.
I have second precision and live in the UTC/GMT/Zulu timezone.
I use ISO/International conventions and protocols only. 

I have an efficient internal representation:

	jnd - the julian day number <SmallInteger>
	secs - the number of seconds since the beginning of the day <SmallInteger>

Examples:

	ZTimestamp now.!

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/21/2012 17:21'!
clockEpoch
	^ self year: 1901 month: 1 day: 1! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/20/2012 22:42'!
clockSeconds
	^ Time totalSeconds! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/20/2012 22:53'!
clockTimezoneOffset
	^ DateAndTime localTimeZone offset asSeconds! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/21/2012 13:07'!
fromClockSeconds: seconds
	^ self new
		jdn: self clockEpoch julianDayNumber + (seconds // 86400)
		secs: self clockEpoch secondsSinceMidnight + (seconds \\ 86400)
	! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:03'!
fromJulianDate: julianDate
	"Create and return a new ZTimestamp instance from the scientific Julian Date, a Float number of Julian Days"
	
	^ self new
		jdn: julianDate truncated
		secs: (julianDate fractionPart * 86400) truncated
		! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:04'!
fromUnixTime: integer
	"Create and return a new ZTimestamp from the integer number of seconds since the Unix or POSIX epoch"
	
	^ self unixEpoch + integer seconds
! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:31'!
jdnFromYear: year month: month day: day
	"Compute and return the Julian Day Number from the Gregorian Calender date described by year, month and day.
	See http://en.wikipedia.org/wiki/Julian_date#Converting_Gregorian_calendar_date_to_Julian_Day_Number"
	 
	| a y m |
	a := (14 - month) // 12.
	y := year + 4800 - a.
	m := month + (12 * a) - 3.
	^ day 
		+ (((153 * m) + 2) // 5) 
		+ (365 * y)
		+ (y // 4)
		- (y // 100)
		+ (y // 400)
		- 32045! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:23'!
localTimeZone
	^ DateAndTime localTimeZone! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:04'!
now
	"Create and return a new ZTimestamp equal to the current system time"
	
	^ self fromClockSeconds: (self clockSeconds - self clockTimezoneOffset)! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 17:55'!
readFrom: stream
	"Parse a ISO 8601 representation from stream, in the format
	year[-]month[-]day[T]hour[:]minute[:]second[[+|-]hh[:]mm]
	where the whole part after T as well as the TZ offset is optional.
	TZ offset can be Z, all separators can be any non digit"

	| year month day hour minute second timestamp offset |
	year := self readInteger: 4 from: stream.
	self readOptionalSeparatorFrom: stream.
	month := self readInteger: 2 from: stream.
	self readOptionalSeparatorFrom: stream.
	day := self readInteger: 2 from: stream.
	(self readOptionalSeparatorFrom: stream)
		ifNotNil: [
			hour := self readInteger: 2 from: stream.
			self readOptionalSeparatorFrom: stream.
			minute := self readInteger: 2 from: stream.
			self readOptionalSeparatorFrom: stream.
			second := self readInteger: 2 from: stream.
			offset := self readTimezoneOffsetFrom: stream ]
		ifNil: [
			hour := minute := second := offset := 0 ].
	timestamp := self 
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second.
	^ offset = 0
		ifTrue: [ timestamp ]
		ifFalse: [ timestamp - offset seconds ]! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/21/2012 17:37'!
readInteger: numberOfDigits from: stream
	| integer |
	integer := 0.
	numberOfDigits timesRepeat: [ | char |
		char := stream next.
		self assert: ('0123456789' includes: char).
		integer := (integer * 10) + char digitValue ].
	^ integer! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/21/2012 17:43'!
readOptionalSeparatorFrom: stream
	^ (stream atEnd or: [ '0123456789' includes: stream peek])
		ifTrue: [ nil ]
		ifFalse: [ stream next ]! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/21/2012 22:13'!
readTimezoneOffsetFrom: stream
	| sign hour minute |
	stream atEnd ifTrue: [ ^ 0 ].
	stream peek = $Z ifTrue: [ stream next. ^ 0 ].
	^ ('+-' includes: stream peek)
		ifTrue: [
			sign := stream next = $- ifTrue: [ -1 ] ifFalse: [ 1 ].
			hour := self readInteger: 2 from: stream.
			(self readOptionalSeparatorFrom: stream)
				ifNil: [ minute := 0 ]
				ifNotNil: [ minute := self readInteger: 2 from: stream ].
			sign * (hour * 3600) + (minute * 60) ]
		ifFalse: [ 0 ]
! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:01'!
unixEpoch
	"Return the Unix or POSIX epoch as a ZTimestamp"
	
	^ self year: 1970 month: 1 day: 1! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:06'!
year: year month: month day: day
	"Create and return a new ZTimestamp for the Gregorian calendar date described by year, month and date"
	
	^ self new
		jdn: (self jdnFromYear: year month: month day: day)
		secs: 0! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:07'!
year: year month: month day: day hour: hour minute: minute second: second
	"Create and return a new ZTimestamp for the Gregorian calendar date described by year, month and date
	and the time of day on that date described by hour, minute and second"

	^ self new
		jdn: (self jdnFromYear: year month: month day: day)
		secs: (hour * 3600) + (minute * 60) + second! !

!ZTimestamp methodsFor: 'arithmetic' stamp: 'SvenVanCaekenberghe 4/21/2012 19:54'!
+ operand
	"Return a new ZTimestamp that is the result of adding
	the operand considered as a Duration to the receiver"
	
	^ self class new
		jdn: jdn
		secs: secs + operand asDuration asSeconds! !

!ZTimestamp methodsFor: 'arithmetic' stamp: 'SvenVanCaekenberghe 4/21/2012 19:56'!
- operand
	"For ZTimestamp compatible operands, return a new Duration equal to the difference between the reveiver and the operand.
	Else return a new ZTimestamp that is the result of subtracting the operand considered as a Duration from the receiver"
	
	^ (operand respondsTo: #asZTimestamp)
		ifTrue: [ | timestamp |
			timestamp := operand asZTimestamp.
			Duration seconds:
				(jdn - timestamp julianDayNumber) * 86400
					+ (secs - timestamp secondsSinceMidnight) ]
		ifFalse: [ self + (operand negated) ]! !

!ZTimestamp methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:31'!
< aZTimestamp
	^ aZTimestamp julianDayNumber = jdn
		ifTrue: [ secs < aZTimestamp secondsSinceMidnight ]
		ifFalse: [ jdn < aZTimestamp julianDayNumber ]! !

!ZTimestamp methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2012 15:05'!
= aZTimestamp
	self == aZTimestamp ifTrue: [ ^ true ].
	(aZTimestamp isKindOf: self class) ifFalse: [ ^ false ].
	^ aZTimestamp julianDayNumber = jdn
		and: [ aZTimestamp secondsSinceMidnight = secs ]! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:08'!
asDate
	^ self asDateAndTime asDate! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:06'!
asDateAndTime 
	^ DateAndTime new
		setJdn: jdn 
		seconds: secs 
		nano: 0 
		offset: 0! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:08'!
asMonth
	^ self asDateAndTime asMonth! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:10'!
asTime
	^ Time seconds: secs! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:06'!
asTimeStamp 
	^ TimeStamp new
		setJdn: jdn 
		seconds: secs 
		nano: 0 
		offset: 0! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:59'!
asUnixTime
	"Return the Integer number of seconds since the Unix or POSIX epoch."
	
	^ (self - self class unixEpoch) asSeconds! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:08'!
asWeek
	^ self asDateAndTime asWeek! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:08'!
asYear
	^ self asDateAndTime asYear! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 13:12'!
asZTimestamp
	^ self! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:50'!
day
	^ self dayOfYear! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:35'!
dayMonthYearDo: block
	"Return the value of executing block with the Gregorian Calender day, month and year as arguments,
	as computed from my Julian Day Number, jdn.
	See http://en.wikipedia.org/wiki/Julian_date#Gregorian_calendar_from_Julian_day_number"
	
	| j g dg c dc b db a da y m d |
	j := jdn + 32044.
	g := j // 146097.
	dg := j \\ 146097.
	c := ((dg // 36524) + 1) * 3 // 4.
	dc := dg - (c * 36524).
	b := dc // 1461.
	db := dc \\ 1461.
	a := ((db // 365) + 1) * 3 // 4.
	da := db - (a * 365).
	y := (g * 400) + (c * 100) + (b * 4) + a.
	m := ((((da * 5) + 308)) // 153) - 2.
	d := da - ((m + 4) * 153 // 5) + 122.
	^ block
		value: d + 1
		value: ((m + 2) \\ 12) + 1
		value: (y - 4800 + ((m * 2) // 12)) ! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 16:59'!
dayOfMonth
	^ self dayMonthYearDo: [ :day :month :year | day ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 17:01'!
dayOfWeek
	^ ((jdn + 1) \\ 7) + 1! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 17:03'!
dayOfWeekName
	^ #(Sunday Monday Tuesday Wednesday Thursday Friday Saterday) at: self dayOfWeek! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:50'!
dayOfYear
	^ self dayMonthYearDo: [ :day :month :year | | monthStart |
		monthStart := #(1 32 60 91 121 152 182 213 244 274 305 335) at: month.
		(month > 2 and: [ Year isLeapYear: year ])
			ifTrue: [ monthStart + day ]
			ifFalse: [ monthStart + day - 1 ] ]! !

!ZTimestamp methodsFor: 'hash' stamp: 'SvenVanCaekenberghe 4/21/2012 20:08'!
hash
	^ jdn bitXor: secs! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:56'!
hour
	^ (secs \\ 86400) // 3600! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:59'!
hours
	^ self hour! !

!ZTimestamp methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/21/2012 13:07'!
jdn: julianDayNumber secs: secondsSinceMidnight
	jdn := julianDayNumber.
	secs := secondsSinceMidnight.
	self normalize! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:00'!
julianDate
	"Return the scientific Julian Date, a Float number of Julian Days"
	
	^ jdn + (secs / 86400.0)! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 15:11'!
julianDayNumber
	^ jdn! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:09'!
localPrintString
	"Convencience method to convert the receiver in an ISO 8601 representation using the local timezone"

	^ String streamContents: [ :stream |
		self printLocalOn: stream ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:58'!
minute
	^ (secs \\ 3600) // 60! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:59'!
minutes
	^ self minute! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 19:56'!
month
	^ self dayMonthYearDo: [ :day :month :year | month ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 17:01'!
monthIndex
	^ self month! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 17:02'!
monthName
	^ #(January Februari March April May June July August September October November December) at: self monthIndex! !

!ZTimestamp methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/21/2012 20:09'!
normalize
	"Normalize the receiver such that 0 <= secs < 86400"
	 
	secs < 86400
		ifFalse: [
			jdn := jdn + (secs // 86400).
			secs := secs \\ 86400 ].
	secs < 0
		ifTrue: [
			jdn := jdn - 1.
			secs := secs + 86400 ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:51'!
offset
	^ 0! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:45'!
printLocalOn: stream
	"Print the receiver on stream in ISO 8601 extended format in the local timezone"
	
	self 
		printOn: stream 
		format: #($- $T $: nil) 
		timezone: self class localTimeZone offset! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:32'!
printOn: stream
	"Print the receiver on stream in ISO 8601 extended format"
	
	self printOn: stream format: #($- $T $: $Z)! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:34'!
printOn: stream format: format
	"Print the receiver on stream using format, following ISO 8601.
	Format defines 4 characters to use:
	{dateSeparator,dateTimeSeparator,timeSeparator,timeZoneSeparator}
	Each can be nil to skip that separator"
	
	self dayMonthYearDo: [ :day :month :year |
		year printOn: stream base: 10 length: 4 padded: true.
		(format at: 1) ifNotNil: [ :separator | stream nextPut: separator ].
		month printOn: stream base: 10 length: 2 padded: true.
		(format at: 1) ifNotNil: [ :separator | stream nextPut: separator ].
		day printOn: stream base: 10 length: 2 padded: true ].
	(format at: 2) ifNotNil: [ :separator | stream nextPut: separator ].
	self hour printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	self minute printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	self second printOn: stream base: 10 length: 2 padded: true.
	(format at: 4) ifNotNil: [ :separator | stream nextPut: separator ]! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:46'!
printOn: stream format: format timezone: timezone
	"Print an ISO representation in timezone and adding it as [+|-]HH:MM"

	self assert: (format at: 4) isNil.
	(self + timezone asDuration) printOn: stream format: format.
	stream nextPut: (timezone positive ifTrue: [ $+ ] ifFalse: [ $- ]).
	timezone hours printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	timezone minutes printOn: stream base: 10 length: 2 padded: true! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:57'!
second
	^ secs \\ 60! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 18:59'!
seconds
	^ self second! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 15:11'!
secondsSinceMidnight
	^ secs! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2012 19:56'!
year
	^ self dayMonthYearDo: [ :day :month :year | year ]! !

!DateAndTime methodsFor: '*ztimestamp' stamp: 'SvenVanCaekenberghe 4/21/2012 19:18'!
asZTimestamp
	^ ZTimestamp new
		jdn: jdn 
		secs: seconds - offset asSeconds! !

TestCase subclass: #ZTimestampTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestampTests commentStamp: 'SvenVanCaekenberghe 4/20/2012 16:54' prior: 0!
I am ZTimestampTests, a test suite for ZnTimestamp.!

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:39'!
assertInvariants: timestamp
	self assert: (timestamp isKindOf: ZTimestamp).
	self assert: timestamp julianDayNumber isInteger.
	self assert: (timestamp secondsSinceMidnight between: 0 and: (24 * 60 * 60) - 1).
	self assert: timestamp offset = 0! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:47'!
assertTimestamp: timestamp equalsTimestamp: reference 
	self assert: reference julianDayNumber = timestamp julianDayNumber.
	self assert: reference secondsSinceMidnight = timestamp secondsSinceMidnight.
	self assert: reference year = timestamp year.
	self assert: reference month = timestamp month.
	self assert: reference dayOfMonth = timestamp dayOfMonth.
	self assert: reference hour = timestamp hour.
	self assert: reference minute = timestamp minute.
	self assert: reference second = timestamp second! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:27'!
testAddingOneDay
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 8 hour: 14 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t1 + 1 day) = t2.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testAddingOneHour
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 15 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t1 + 1 hour) = t2.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:48'!
testBasicFieldAccessors
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp year = 2012.
	self assert: timestamp month = 4.
	self assert: timestamp dayOfMonth = 20.
	self assert: timestamp hour = 15.
	self assert: timestamp minute = 50.
	self assert: timestamp seconds = 10	! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:45'!
testCreationWithAllFields
	| reference timestamp |
	reference := DateAndTime 
		year: 2012 
		month: 4 
		day: 20 
		hour: 15 
		minute: 50 
		second: 10 
		nanoSecond: 0 
		offset: 0.
	timestamp := ZTimestamp 
		year: reference year 
		month: reference month 
		day: reference dayOfMonth 
		hour: reference hour 
		minute: reference minute 
		second: reference second.
	self assertTimestamp: reference equalsTimestamp: timestamp! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:45'!
testCreationWithDateFields
	| reference timestamp |
	reference := DateAndTime 
		year: 2012 
		month: 4 
		day: 20 
		hour: 0 
		minute: 0 
		second: 0 
		nanoSecond: 0 
		offset: 0.
	timestamp := ZTimestamp 
		year: reference year 
		month: reference month 
		day: reference dayOfMonth.
	self assertTimestamp: reference equalsTimestamp: timestamp! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:25'!
testEquality
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	self assert: t1 = t2.
	self assert: ZTimestamp now = ZTimestamp now.
	self assert: ZTimestamp unixEpoch = ZTimestamp unixEpoch.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:54'!
testLocalPrintString
	| timestamp reference |
	timestamp := ZTimestamp now localPrintString.
	reference := DateAndTime now truncated printString.
	self assert: timestamp = reference! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:40'!
testNow
	self assertInvariants: ZTimestamp now! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testOneDayDifference
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 8 hour: 14 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t2 - t1) = 1 day! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testOneHourDifference
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 15 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t2 - t1) = 1 hour.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 22:09'!
testParsing
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11Z' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '20120807T142011' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11+00:00' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T16:20:11+02:00' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T12:20:11-02:00' readStream) = timestamp.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 22:14'!
testParsingDates
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 8 day: 7.
	self assert: (ZTimestamp readFrom: '2012-08-07' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '20120807' readStream) = timestamp.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2012 21:35'!
testPrintString
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp printString = '2012-04-20T15:50:10Z'! !
